@@ -1,13 +1,18 @@
+from abc import abstractmethod
 from functools import partial
 import operator as op
 import io
 import numpy as np
 from itertools import combinations_with_replacement, repeat, chain
 
+from typing import Tuple, Optional, Union, TypeVar
+
 from . import raw
 
 
-__version__ = '1.7.0'
+__version__ = '1.10.0'
+
+T = TypeVar('T')
 
 
 def _ensure_listlike(x, dups=1):
@@ -95,10 +100,21 @@ def _derivative_helper(pts, derivs, func):
     """
 
     singlept = not isinstance(pts[0], np.ndarray)
-    singlederiv = isinstance(derivs[0], int)
 
-    if singlederiv:
+    # If requesting a single derivative
+    if isinstance(derivs[0], int):
         nderiv, index = _derivative_index(derivs)
+        
+        # If requesting a single point
+        if singlept:
+            return func(*pts, nderiv)[index]
+        
+        # If requesting multiple points
+        else:
+            data = [func(*pt, nderiv) for pt in zip(*pts)]
+            return np.array([d[index] for d in data])
+        
+    # If requesting several derivatives
     else:
         nderiv, indexes = 0, []
         for deriv in derivs:
@@ -106,17 +122,14 @@ def _derivative_helper(pts, derivs, func):
             nderiv = max(n, nderiv)
             indexes.append(i)
 
-    if singlept:
-        data = func(*pts, nderiv)
-        if singlederiv:
-            return data[index]
-        else:
+        # If requesting a single point
+        if singlept:
+            data = func(*pts, nderiv)
             return np.array([data[i] for i in indexes])
-    else:
-        data = [func(*pt, nderiv) for pt in zip(*pts)]
-        if singlederiv:
-            return np.array([d[index] for d in data])
+        
+        # If requesting multiple points
         else:
+            data = [func(*pt, nderiv) for pt in zip(*pts)]
             return np.array([[d[i] for d in data] for i in indexes])
 
 
@@ -211,6 +224,9 @@ class Element(SimpleWrapper):
         self.lr.w.refineElement(self.id)
         self.lr.w.generateIDs()
 
+    def bezier_extraction(self):
+        return self.lr.bezier_extraction(self.id)
+
     def __eq__(self, other):
         if not isinstance(other, Element):
             return False
@@ -218,6 +234,14 @@ class Element(SimpleWrapper):
 
 
 class MeshInterface(SimpleWrapper):
+    
+    constant_direction: int
+    value: float
+    variable_directions: Tuple[int, ...]
+    
+    @abstractmethod
+    def span(self, direction: Optional[Union[int, str]] = None) -> Tuple[float, float]:
+        pass
 
     def __str__(self):
         cls = self.__class__.__name__
@@ -358,20 +382,25 @@ class MeshRectView(ListLikeView):
 
 
 class LRSplineObject:
+    
+    @abstractmethod
+    def clone(self: T) -> T:
+        pass
 
-    def __init__(self, w):
-        w.generateIDs()
+    def __init__(self, w, renumber=True):
+        if renumber:
+            w.generateIDs()
         self.w = w
         self.elements = ElementView(self)
         self.basis = BasisView(self)
 
     @staticmethod
-    def read_many(stream):
+    def read_many(stream, renumber=True):
         contents = stream.read()
         splitter = b'# LRSPLINE' if isinstance(contents, bytes) else '# LRSPLINE'
         contents = contents.split(splitter)[1:]
         contents = [splitter + c for c in contents]
-        patches = [_constructor(c)(c) for c in contents]
+        patches = [_constructor(c)(c, renumber=renumber) for c in contents]
         return patches
 
     def __len__(self):
@@ -477,6 +506,15 @@ class LRSplineObject:
     def generate_ids(self):
         self.w.generateIDs()
 
+    def cache_elements(self):
+        self.w.createElementCache()
+
+    def bezier_extraction(self, arg):
+        if isinstance(arg,Element):
+            return self.w.getBezierExtraction(arg.id)
+        else: # isinstance(arg,int)
+            return self.w.getBezierExtraction(arg)
+
     def __mul__(self, x):
         new = self.clone()
         new.controlpoints *= x
@@ -491,7 +529,7 @@ class LRSplineObject:
 
 class LRSplineSurface(LRSplineObject):
 
-    def __init__(self, *args):
+    def __init__(self, *args, renumber=True):
         if len(args) == 0:
             w = raw.LRSurface()
         elif isinstance(args[0], raw.LRSurface):
@@ -510,7 +548,7 @@ class LRSplineSurface(LRSplineObject):
             w = raw.LRSurface(args[0], args[1], args[2], args[3], args[4], args[5], cp.flat, len(cp[0]))
         else:
             w = raw.LRSurface()
-        super().__init__(w)
+        super().__init__(w, renumber=renumber)
         self.meshlines = MeshLineView(self)
 
     def corners(self):
@@ -575,18 +613,18 @@ class LRSplineSurface(LRSplineObject):
         wrapper = lambda u,v,n: self.w.point(u, v, n, iEl=iel)
         return _derivative_helper((u, v), d, wrapper)
 
-    def bezier_extraction(self, iEl):
-        return self.w.getBezierExtraction(iEl)
-
     def element_at(self, u, v):
         return self.elements[self.w.getElementContaining(u,v)]
 
+    def make_integer_knots(self):
+        return self.w.makeIntegerKnots()
+
     __call__ = evaluate
 
 
 class LRSplineVolume(LRSplineObject):
 
-    def __init__(self, *args):
+    def __init__(self, *args, renumber=True):
         if len(args) == 0:
             w = raw.LRVolume()
         elif isinstance(args[0], raw.LRVolume):
@@ -605,7 +643,7 @@ class LRSplineVolume(LRSplineObject):
             w = raw.LRVolume(*args[:-1], cp.flat, len(cp[0]))
         else:
             w = raw.LRVolume()
-        super().__init__(w)
+        super().__init__(w, renumber=renumber)
         self.meshrects = MeshRectView(self)
 
     def corners(self):
@@ -635,7 +673,7 @@ class LRSplineVolume(LRSplineObject):
         self.w.generateIDs()
 
     def insert(self, mr):
-        self.w.insert_line(mr.w.copy())
+        self.w.insert_line(mr.w)
 
     def evaluate(self, u, v, w, iel=-1):
         if isinstance(u, np.ndarray) and isinstance(v, np.ndarray) and isinstance(w, np.ndarray):
@@ -643,9 +681,6 @@ class LRSplineVolume(LRSplineObject):
             return retval.reshape(u.shape + (-1,))
         return self.w.point(u, v, w, iEl=iel)
 
-    def bezier_extraction(self, iEl):
-        return self.w.getBezierExtraction(iEl)
-
     def element_at(self, u, v, w):
         return self.elements[self.w.getElementContaining(u,v,w)]
 
